#!/usr/bin/env python

import numpy as np
from matplotlib import pyplot as plt
from matplotlib.backends.qt_compat import QtWidgets
from matplotlib.widgets import Slider, Button

N = 5
N_MAX = 100

NN = np.arange(0, N_MAX + 1)

tt = np.linspace(0., 2.5, 2501)

ysq = 1.0 * (tt % 1.0) < 0.3
fsq = 0.5j / (np.pi * NN) * (np.exp(-0.6j * np.pi * NN) - 1.0)
fsq[0] = 0.3

ytr = 2. * np.abs((tt % 1.0) - 0.5)
ftr = 2. / (np.pi * NN)**2
ftr[NN % 2 == 0] = 0.
ftr[0] = 0.5

yts = (tt % 1.0)
fts = 0.5j / (np.pi * NN)
fts[0] = 0.5


def ifft(c, n=N):
    c = c[:n + 1]
    c = np.concatenate((c[-1:0:-1].conj(), c))
    nn = np.arange(-n, n + 1)
    return np.sum(c.reshape(-1, 1) * np.exp(2j * np.pi * nn.reshape(-1, 1) * tt.reshape(1, -1)), axis=0)


fig, (ax1, ax2) = plt.subplots(nrows=2, sharex=True, gridspec_kw={'hspace': 0.05})

manager = fig.canvas.manager
manager.window.findChild(QtWidgets.QToolBar).setVisible(False)
manager.set_window_title("Szeregi Fouriera")
manager.full_screen_toggle()

data = ytr
coeffs = ftr

fourier, = ax1.plot(tt, ifft(coeffs, N), color='m', lw=1.5)
original, = ax1.plot(tt, data, color='k', alpha=0.4, lw=1.5)

cmapc = plt.get_cmap('autumn')
cmaps = plt.get_cmap('winter')
colsc = [cmapc(i / N) for i in NN]
colss = [cmaps(i / N) for i in NN]
coss = [ax2.plot(tt, coeffs[i].real * np.cos(2 * np.pi * i * tt), color=colsc[i], lw=0.5)[0] for i in NN]
sins = [ax2.plot(tt, coeffs[i].imag * np.sin(2 * np.pi * i * tt), color=colss[i], ls='--', lw=0.5)[0] for i in NN]

for n in range(N + 1, N_MAX + 1):
    coss[n].set_visible(False)
    sins[n].set_visible(False)

ax2.set_xlim(tt[0], tt[-1])
ax2.set_xlabel('$t / T$')

ax1.set_ylabel('Rozwijana funkcja')
ax2.set_ylabel("Współczynniki")

fig.subplots_adjust(bottom=0.15, top=0.98, left=0.08, right=0.98)
nax = fig.add_axes([0.55, 0.03, 0.40, 0.03])

sqax = fig.add_axes([0.02, 0.01, 0.10, 0.07])
trax = fig.add_axes([0.14, 0.01, 0.10, 0.07])
tsax = fig.add_axes([0.26, 0.01, 0.10, 0.07])

sst = {
    'track_color': '0.7',
    'color': '0.7',
    'initcolor': 'white',
}

N_slider = Slider(ax=nax, label='Ilość współczynników', valmin=0, valmax=N_MAX, valstep=1, valinit=N, **sst)

tr_button = Button(ax=sqax, label='Trójkątna')
sq_button = Button(ax=trax, label='Kwadratowa')
ts_button = Button(ax=tsax, label='Piłokształtna')


def update_fourier(n):
    original.set_ydata(data)
    fourier.set_ydata(ifft(coeffs, n))
    colsc = [cmapc(i / n) for i in range(n + 1)] if n != 0 else [cmapc(0)]
    colss = [cmaps(i / n) for i in range(n + 1)] if n != 0 else [cmaps(0)]
    for i in range(0, n + 1):
        coss[i].set_visible(True)
        sins[i].set_visible(True)
        coss[i].set_color(colsc[i])
        sins[i].set_color(colss[i])
    for i in range(n + 1, N_MAX + 1):
        coss[i].set_visible(False)
        sins[i].set_visible(False)
    ax1.relim()
    ax1.autoscale_view()
    ax2.relim()
    ax2.autoscale_view()
    fig.canvas.draw_idle()


def change_data(dat, cfs):
    global data, coeffs
    data = dat
    coeffs = cfs
    for i in range(N_MAX + 1):
        coss[i].set_ydata(coeffs[i].real * np.cos(2 * np.pi * i * tt))
        sins[i].set_ydata(coeffs[i].imag * np.sin(2 * np.pi * i * tt))
    update_fourier(N_slider.val)


sq_button.on_clicked(lambda _: change_data(ysq, fsq))
tr_button.on_clicked(lambda _: change_data(ytr, ftr))
ts_button.on_clicked(lambda _: change_data(yts, fts))
N_slider.on_changed(update_fourier)

plt.show()
